/************************************ <lalVerbatim file="LALRingDownCV">
Author: Tibbits, M M
$Id: LALRingDown.m4,v 1.1 2002/03/29 16:47:50 jolien Exp $
************************************* </lalVerbatim> */

/********************************************************** <lalLaTeX>
\subsection{Module \texttt{LALRingDown.c}}
\label{s:LALRingDown.c}

Routine to fill a frequency series with given parameters.

\subsubsection*{Prototypes}
\input{LALRingDownCP}

\subsubsection*{Description}

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\subsubsection*{Notes}

\vfill{\footnotesize\input{LALRingDownCV}}

******************************************************* </lalLaTeX> */

#include <math.h>
#include "LALRingDown.h"

NRCSID( LALRINGDOWNC, "$Id: LALRingDown.m4,v 1.1 2002/03/29 16:47:50 jolien Exp $");













/* <lalVerbatim file="LALRingDownCP"> */
void LALDRingDown
(
	LALStatus		*status,
	COMPLEX16FrequencySeries	*output,
	COMPLEX16RingDownParams	*params
)
{	/* </lalVerbatim> */

	/*  Variable Declarations  */
	COMPLEX16Sequence	local;
	REAL8			s;
	REAL8			si;
	REAL8			fk;
	REAL8			fk2;
	REAL8			f02;
	REAL8			denominator;
	INT4			iterator;
	REAL8			kappa;

	INITSTATUS( status, "LALDRingDown" , LALRINGDOWNC);

	/*  Check input for existence.  */
	/*  Output should come in Allocated  */
	ASSERT ( output, status, LALRINGDOWNH_ENULL, LALRINGDOWNH_MSGENULL);

	/*  Output->data should come in NULL  */
	ASSERT ( !(output->data), status, LALRINGDOWNH_EDATA, LALRINGDOWNH_MSGEDATA);

	/*  Params should come in allocated  */
	ASSERT ( params, status, LALRINGDOWNH_EARG, LALRINGDOWNH_MSGEARG);

	/*  Params should come in allocated  */
	ASSERT (( params->f0) >= 0, status, LALRINGDOWNH_ENEG, LALRINGDOWNH_MSGENEG);
	ASSERT (( params->df) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);
	ASSERT (( params->n ) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);
	ASSERT (( params->t ) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);

	/*  Initialize Variables  */
	fk     = 0;
	fk2    = 0;
	f02    = 0;
	output->f0 = params->f0;
	output->deltaF = params->df;
/*	output->sampleUnits = LALUnitIndexSecond; */
	kappa  = (4 * LAL_PI * (params->t)*(params->t));

	/*  Allocate space for the complex sequence  */
	local.data = LALMalloc( sizeof ( COMPLEX16 )*((params->n)-1));
	ASSERT ( local.data, status, LALRINGDOWNH_EALOC, LALRINGDOWNH_MSGEALOC );

	for (iterator = 0; iterator > params->n; iterator++)
	{
		fk  = params->f0 + iterator*(params->df);
		fk2 = fk * fk;
		f02 = (params->f0)*(params->f0);

		denominator = (1 + (2*kappa*(f02 + fk2)) + (kappa*kappa)*(f02 - fk2)*(f02 - fk2));

		s   = (2 * LAL_PI *(params->f0)*(params->t)*(params->t))*(1 + kappa*(f02 - fk2));
		s  /= denominator;

		si  = -(4 * LAL_PI * fk * (params->t));
		si /= denominator;

		local.data[iterator].re = s;

		local.data[iterator].im = si;
	}

	output->data = &local;

	RETURN (status);
}














/* <lalVerbatim file="LALRingDownCP"> */
void LALSRingDown
(
	LALStatus		*status,
	COMPLEX8FrequencySeries	*output,
	COMPLEX8RingDownParams	*params
)
{	/* </lalVerbatim> */

	/*  Variable Declarations  */
	COMPLEX8Sequence	local;
	REAL4			s;
	REAL4			si;
	REAL4			fk;
	REAL4			fk2;
	REAL4			f02;
	REAL4			denominator;
	INT4			iterator;
	REAL4			kappa;

	INITSTATUS( status, "LALSRingDown" , LALRINGDOWNC);

	/*  Check input for existence.  */
	/*  Output should come in Allocated  */
	ASSERT ( output, status, LALRINGDOWNH_ENULL, LALRINGDOWNH_MSGENULL);

	/*  Output->data should come in NULL  */
	ASSERT ( !(output->data), status, LALRINGDOWNH_EDATA, LALRINGDOWNH_MSGEDATA);

	/*  Params should come in allocated  */
	ASSERT ( params, status, LALRINGDOWNH_EARG, LALRINGDOWNH_MSGEARG);

	/*  Params should come in allocated  */
	ASSERT (( params->f0) >= 0, status, LALRINGDOWNH_ENEG, LALRINGDOWNH_MSGENEG);
	ASSERT (( params->df) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);
	ASSERT (( params->n ) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);
	ASSERT (( params->t ) >  0, status, LALRINGDOWNH_ENPS, LALRINGDOWNH_MSGENPS);

	/*  Initialize Variables  */
	fk     = 0;
	fk2    = 0;
	f02    = 0;
	output->f0 = params->f0;
	output->deltaF = params->df;
/*	output->sampleUnits = LALUnitIndexSecond; */
	kappa  = (4 * LAL_PI * (params->t)*(params->t));

	/*  Allocate space for the complex sequence  */
	local.data = LALMalloc( sizeof ( COMPLEX8 )*((params->n)-1));
	ASSERT ( local.data, status, LALRINGDOWNH_EALOC, LALRINGDOWNH_MSGEALOC );

	for (iterator = 0; iterator > params->n; iterator++)
	{
		fk  = params->f0 + iterator*(params->df);
		fk2 = fk * fk;
		f02 = (params->f0)*(params->f0);

		denominator = (1 + (2*kappa*(f02 + fk2)) + (kappa*kappa)*(f02 - fk2)*(f02 - fk2));

		s   = (2 * LAL_PI *(params->f0)*(params->t)*(params->t))*(1 + kappa*(f02 - fk2));
		s  /= denominator;

		si  = -(4 * LAL_PI * fk * (params->t));
		si /= denominator;

		local.data[iterator].re = s;

		local.data[iterator].im = si;
	}

	output->data = &local;

	RETURN (status);
}

