/* -*- C -*- */
/******** <lalVerbatim file="VectorIndexRangeCV"> ********
Author: David Chin <dwchin@umich.edu> +1-734-709-9119
$Id: VectorIndexRangeC.m4,v 1.2 2003/08/14 15:09:15 dwchin Exp $
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\subsection{Module \texttt{VectorIndexRange.c}}
\label{s:VectorIndexRange.c}


\subsection*{Description}

\subsection*{Algorithms}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LAL*CreateVector}
\end{itemize}

\subsection*{Notes}

\subsubsection*{Prototypes}

 *************************************************** </lalLaTeX> */

#include <math.h>
#include "VectorIndexRange.h"

NRCSID( VECTORINDEXRANGEC, "$Id: VectorIndexRangeC.m4,v 1.2 2003/08/14 15:09:15 dwchin Exp $" );


/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALCHARVectorIndexRange ( LALStatus *status, CHARVector **result,
       CHARVector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALCHARVectorIndexRange ( 
         LALStatus  *status,
         CHARVector     **p_result,
         CHARVector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALCHARVectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALCHARDestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALCHARCreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALCHARVectorIndexHole ( LALStatus  *status,
          CHARVectorPair  *p_result_pair,
          CHARVector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALCHARVectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALCHARVectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALCHARVectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALCHARVectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALCHARDestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALI2VectorIndexRange ( LALStatus *status, INT2Vector **result,
       INT2Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALI2VectorIndexRange ( 
         LALStatus  *status,
         INT2Vector     **p_result,
         INT2Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALI2VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALI2DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALI2CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALI2VectorIndexHole ( LALStatus  *status,
          INT2VectorPair  *p_result_pair,
          INT2Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALI2VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALI2VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALI2VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALI2VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALI2DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALI4VectorIndexRange ( LALStatus *status, INT4Vector **result,
       INT4Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALI4VectorIndexRange ( 
         LALStatus  *status,
         INT4Vector     **p_result,
         INT4Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALI4VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALI4DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALI4CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALI4VectorIndexHole ( LALStatus  *status,
          INT4VectorPair  *p_result_pair,
          INT4Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALI4VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALI4VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALI4VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALI4VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALI4DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALI8VectorIndexRange ( LALStatus *status, INT8Vector **result,
       INT8Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALI8VectorIndexRange ( 
         LALStatus  *status,
         INT8Vector     **p_result,
         INT8Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALI8VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALI8DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALI8CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALI8VectorIndexHole ( LALStatus  *status,
          INT8VectorPair  *p_result_pair,
          INT8Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALI8VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALI8VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALI8VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALI8VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALI8DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALU2VectorIndexRange ( LALStatus *status, UINT2Vector **result,
       UINT2Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALU2VectorIndexRange ( 
         LALStatus  *status,
         UINT2Vector     **p_result,
         UINT2Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALU2VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALU2DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALU2CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALU2VectorIndexHole ( LALStatus  *status,
          UINT2VectorPair  *p_result_pair,
          UINT2Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALU2VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALU2VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALU2VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALU2VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALU2DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALU4VectorIndexRange ( LALStatus *status, UINT4Vector **result,
       UINT4Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALU4VectorIndexRange ( 
         LALStatus  *status,
         UINT4Vector     **p_result,
         UINT4Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALU4VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALU4DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALU4CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALU4VectorIndexHole ( LALStatus  *status,
          UINT4VectorPair  *p_result_pair,
          UINT4Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALU4VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALU4VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALU4VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALU4VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALU4DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALU8VectorIndexRange ( LALStatus *status, UINT8Vector **result,
       UINT8Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALU8VectorIndexRange ( 
         LALStatus  *status,
         UINT8Vector     **p_result,
         UINT8Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALU8VectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALU8DestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALU8CreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALU8VectorIndexHole ( LALStatus  *status,
          UINT8VectorPair  *p_result_pair,
          UINT8Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALU8VectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALU8VectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALU8VectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALU8VectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALU8DestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALSVectorIndexRange ( LALStatus *status, REAL4Vector **result,
       REAL4Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALSVectorIndexRange ( 
         LALStatus  *status,
         REAL4Vector     **p_result,
         REAL4Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALSVectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALSDestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALSCreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALSVectorIndexHole ( LALStatus  *status,
          REAL4VectorPair  *p_result_pair,
          REAL4Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALSVectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALSVectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALSVectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALSVectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALSDestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALDVectorIndexRange ( LALStatus *status, REAL8Vector **result,
       REAL8Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALDVectorIndexRange ( 
         LALStatus  *status,
         REAL8Vector     **p_result,
         REAL8Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALDVectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALDDestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALDCreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALDVectorIndexHole ( LALStatus  *status,
          REAL8VectorPair  *p_result_pair,
          REAL8Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALDVectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALDVectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALDVectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALDVectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALDDestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALCVectorIndexRange ( LALStatus *status, COMPLEX8Vector **result,
       COMPLEX8Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALCVectorIndexRange ( 
         LALStatus  *status,
         COMPLEX8Vector     **p_result,
         COMPLEX8Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALCVectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALCDestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALCCreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALCVectorIndexHole ( LALStatus  *status,
          COMPLEX8VectorPair  *p_result_pair,
          COMPLEX8Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALCVectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALCVectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALCVectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALCVectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALCDestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}



/** -*- C -*- **/
/*















*/

/******************************* <lalLaTeX file="VectorIndexRangeC">
\begin{verbatim}void LALZVectorIndexRange ( LALStatus *status, COMPLEX16Vector **result,
       COMPLEX16Vector *A, const UINT4Vector *indexVector )\end{verbatim}
*************************************************** </lalLaTeX> */

/* Take a vector, and remove a range entries by given indices, then stuff
   that truncated vector in to the result vector */
void LALZVectorIndexRange ( 
         LALStatus  *status,
         COMPLEX16Vector     **p_result,
         COMPLEX16Vector      *p_v,
         const UINT4Vector *p_indexVector
         )
{
  /*  Variable Declarations  */
  UINT4 iter;
  UINT4 rslt_length;

  if (lalDebugLevel & 15)
    {
      printf("GROUCHO:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  INITSTATUS( status, "LALZVectorIndexRange" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );

  if (lalDebugLevel & 15)
    {
      printf("  p_v->length = %d\n", p_v->length);
      printf("  p_indexVector->data[1] = %d\n", p_indexVector->data[1]);
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /*
   * error-checking assertions
   */
  ASSERT(p_v, status, VECTORINDEXRANGEH_ENULL, VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_v->length >= 1, status, VECTORINDEXRANGEH_ELNTH, 
         VECTORINDEXRANGEH_MSGELNTH);

  ASSERT(p_indexVector, status, VECTORINDEXRANGEH_ENULL, 
         VECTORINDEXRANGEH_MSGENULL);
  ASSERT(p_indexVector->length == 2, status, VECTORINDEXRANGEH_ELNTH,
         VECTORINDEXRANGEH_MSGELNTH);

  rslt_length = p_indexVector->data[1] - p_indexVector->data[0] + 1;

  if (lalDebugLevel & 8)
    {
      printf("  rslt_length = %d\n", rslt_length);
      printf("  BEFORE ALLOC:  p_result = %#x\n", (unsigned int)p_result);
      printf("                *p_result = %#x\n", (unsigned int)(*p_result));

      if (lalDebugLevel & 15)
        {
          printf("CHICO:\n");
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* Handle memory */
  if ((*p_result != NULL) && ((*p_result)->length != rslt_length))
    {
      TRY(LALZDestroyVector(status->statusPtr, p_result),
          status);
    }

  if (lalDebugLevel & 15)
    {
      printf("FOOBAR:\n");
      printf("  status = %#x\n", (unsigned int)status);
      printf("  status->statusPtr = %#x\n", (unsigned int)(status->statusPtr));
    }

  /* allocate */
  TRY(LALZCreateVector(status->statusPtr, p_result, rslt_length),
      status);


  if (lalDebugLevel & 8)
    {
      printf("  AFTER ALLOC: p_result = %#x\n", (unsigned int)p_result);
      printf("              *p_result = %#x\n", (unsigned int)(*p_result));
      printf("             (*p_result)->length = %d\n", (*p_result)->length);

      if (lalDebugLevel & 15)
        {
          printf("  status = %#x\n", (unsigned int)status);
          printf("  status->statusPtr = %#x\n",
                 (unsigned int)(status->statusPtr));
        }
    }

  /* fill in the result vector */
  for (iter = p_indexVector->data[0]; iter <= p_indexVector->data[1]; ++iter)
    {
      (*p_result)->data[iter - p_indexVector->data[0]] = p_v->data[iter];
    }
  
  DETATCHSTATUSPTR( status );
  RETURN (status);
}



void LALZVectorIndexHole ( LALStatus  *status,
          COMPLEX16VectorPair  *p_result_pair,
          COMPLEX16Vector      *p_v,
          const UINT4Vector *p_indexVector )
{
  /*  Variable Declarations  */
  UINT4Vector *head_index_range = NULL;
  UINT4Vector *tail_index_range = NULL;

  INITSTATUS( status, "LALZVectorIndexHole" , VECTORINDEXRANGEC );
  ATTATCHSTATUSPTR( status );



  /*
    assert indexVector range
    p_result_pair->head = VectorIndexRange(..., [0,indexVector->data[0]-1]);
    p_result_pair->tail = VectorIndexRange(..., [indexVector->data[1]+1,
                          p_v->length-1])
  */

  TRY(LALU4CreateVector(status->statusPtr, &head_index_range, 2),
      status);

  LALU4CreateVector(status->statusPtr, &tail_index_range, 2);

  BEGINFAIL(status)
    {
      TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
    }
  ENDFAIL(status);

  head_index_range->data[0] = 0;
  head_index_range->data[1] = p_indexVector->data[0] - 1;
  tail_index_range->data[0] = p_indexVector->data[1] + 1;
  tail_index_range->data[1] = p_v->length - 1;

  if (lalDebugLevel & 8)
    {
      printf("LALZVectorIndexHole:\n");
      printf("  head_index_range->length = %d\n", head_index_range->length);
      printf("  head_index_range->data[0] = %d\n",head_index_range->data[0]);
      printf("  head_index_range->data[1] = %d]n",head_index_range->data[1]);

      printf("  tail_index_range->length = %d\n",tail_index_range->length);
      printf("  tail_index_range->data[0] = %d\n",tail_index_range->data[0]);
      printf("  tail_index_range->data[1] = %d]n",tail_index_range->data[1]);
    }

  TRY(LALZVectorIndexRange(status->statusPtr, p_result_pair->head,
                                      p_v, head_index_range), status);
  
  LALZVectorIndexRange(status->statusPtr, p_result_pair->tail,
                                  p_v, tail_index_range);
  BEGINFAIL(status)
    TRY(LALZDestroyVector(status->statusPtr, p_result_pair->head),
        status);
  ENDFAIL(status);

  TRY(LALU4DestroyVector(status->statusPtr, &head_index_range), status);
  TRY(LALU4DestroyVector(status->statusPtr, &tail_index_range), status);

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


/******************************* <lalLaTeX file="VectorIndexRangeC">

\vfill{\footnotesize\input{VectorIndexRangeCV}}

 *************************************************** </lalLaTeX> */
